%%% Econ 203C (Spring 2009)
%%% Problem Set IV, Question 2
%%% Monte Carlo Simulation of J-statistic

clear all;              % clear all the variables
clc;                    % clear the screen

% Set primitive variables
R=1000;                 % number of Monte Carlo simulation
n=100;                  % sample size in each simulation
beta=0;                 % true parameter to be estimated
K=[2,10];               % number of instrumental variables
eta=[.05,1];            % strength of instruments
rho=.3;                 % endogeneity of regressor
delta=[-.3:.05:-.05,-.03:.01:.03,.05:.05:.3];
                        % validity of instruments (delta=0 indicates instruments are valid)

% Set critical values
cv=chi2inv(.95,K-1);    % critical values from chi2 distribution with K-1 degrees of freedom
cv=kron(cv,ones(1,length(eta)*length(delta)));

% Initialize the sequence of random numbers
randn('seed',2032009);

% Initialize the estimates matrix
beta1_est = [];         % 2SLS estimator: Case I
beta2_est = [];         % 2SLS estimator: Case II
J1_sim = [];            % J-statistic
J2_sim = [];
J1_rej = [];            % J-test rejection
J2_rej = [];
Wald1_sim = [];         % Wald statistic
Wald2_sim = [];

% Start iteration
for r=1:R
    % Initialize the intermediate estimates matrix
    beta1_temp = [];
    beta2_temp = [];
    J1_temp = [];
    J2_temp = [];
    Wald1_temp = [];
    Wald2_temp = [];
    
    for h=1:length(K)
        % Generate random matrix used through all the eta's and the rho's
        % standard normal random vector with var=I
        V = randn(n,K(h)+2);
        
        for i=1:length(eta)
            % Form pi: intermediate parameter
            pi = eta(i) * ones(K(h),1);
            
            for j=1:length(delta)
                %%%%%%%%%% Case I %%%%%%%%%%
                % Form Delta: intermediate parameters
                c = delta(j) * ones(K(h),1);
                Delta = [eye(K(h)),c,zeros(K(h),1); ...
                    c',1,rho;...
                    zeros(1,K(h)),rho,1];

                % Data Generating Process
                U = chol(Delta);    % Cholesky decomposition of Delta=U'U
                W = V*U;            % normal random vector with mean=0, var=Delta=U'U
                z = W(:,1:K(h));
                x = z*pi + W(:,K(h)+2);
                y = x*beta + W(:,K(h)+1);
    
                % 2SLS Estimation
                beta1_hat = inv(x'*z*inv(z'*z)*z'*x)*x'*z*inv(z'*z)*z'*y;
                
                % J-statistic
                u_hat = y - x * beta1_hat;
                Omega_hat = (u_hat * ones(1,K(h)) .* z)' * (u_hat * ones(1,K(h)) .* z) / n;
                J1_hat = n * u_hat'*z/n * inv(Omega_hat) * z'*u_hat/n;
%                J1_hat = u_hat'*z*inv(z'*z)*z'*u_hat / (u_hat'*u_hat/n);
                
                % Wald statistic
                sigma2_hat = u_hat'*u_hat / n;
                Wald1_hat = (beta1_hat - beta)'*(x'*z*inv(z'*z)*z'*x)*(beta1_hat - beta) / sigma2_hat;
                
                % Stack estimates to temporary space
                beta1_temp = [beta1_temp,beta1_hat];
                J1_temp = [J1_temp,J1_hat];
                Wald1_temp = [Wald1_temp,Wald1_hat];

                %%%%%%%%%% Case II %%%%%%%%%%
                % Form Delta: intermediate parameters
                c = zeros(K(h),1);
                c(1,1) = delta(j);
                Delta = [eye(K(h)),c,zeros(K(h),1); ...
                    c',1,rho;...
                    zeros(1,K(h)),rho,1];

                % Data Generating Process
                U = chol(Delta);    % Cholesky decomposition of Delta=U'U
                W = V*U;            % normal random vector with mean=0, var=Delta=U'U
                z = W(:,1:K(h));
                x = z*pi + W(:,K(h)+2);
                y = x*beta + W(:,K(h)+1);
    
                % 2SLS Estimation
                beta2_hat = inv(x'*z*inv(z'*z)*z'*x)*x'*z*inv(z'*z)*z'*y;
                
                % J-statistic
                u_hat = y - x * beta2_hat;
                Omega_hat = (u_hat * ones(1,K(h)) .* z)' * (u_hat * ones(1,K(h)) .* z) / n;
                J2_hat = n * u_hat'*z/n * inv(Omega_hat) * z'*u_hat/n;
%                J2_hat = u_hat'*z*inv(z'*z)*z'*u_hat / (u_hat'*u_hat/n);
                
                % Wald statistic
                sigma2_hat = u_hat'*u_hat / n;
                Wald2_hat = (beta2_hat - beta)'*(x'*z*inv(z'*z)*z'*x)*(beta2_hat - beta) / sigma2_hat;
                
                % Stack estimates to temporary space
                beta2_temp = [beta2_temp,beta2_hat];
                J2_temp = [J2_temp,J2_hat];
                Wald2_temp = [Wald2_temp,Wald2_hat];
            end
        end
    end
    
    % J-test
    J1_rej_temp = J1_temp > cv;
    J2_rej_temp = J2_temp > cv;
    
    % Stack estimates for later use
    beta1_est = [beta1_est;beta1_temp];
    beta2_est = [beta2_est;beta2_temp];
    J1_sim = [J1_sim;J1_temp];
    J2_sim = [J2_sim;J2_temp];
    J1_rej = [J1_rej;J1_rej_temp];
    J2_rej = [J2_rej;J2_rej_temp];
    Wald1_sim = [Wald1_sim;Wald1_temp];
    Wald2_sim = [Wald2_sim;Wald2_temp];
end

% Wald test provided the pretest is not rejected
Wald1_rej = Wald1_sim > chi2inv(.95,1) & J1_rej == 0;
Wald2_rej = Wald2_sim > chi2inv(.95,1) & J2_rej == 0;

% Question (ii) Calculate the conditional Wald rejection probability
Wald1_rej_prob = sum(Wald1_rej) ./ sum(1-J1_rej);
Wald2_rej_prob = sum(Wald2_rej) ./ sum(1-J2_rej);

% Question (i) Calculate the J-test rejection probability
J1_rej_prob = mean(J1_rej);
J2_rej_prob = mean(J2_rej);

% Reshape the result in the form interpreted easily
J1_rej_prob = reshape(J1_rej_prob, length(delta), length(K)*length(eta))';
J2_rej_prob = reshape(J2_rej_prob, length(delta), length(K)*length(eta))';
Wald1_rej_prob = reshape(Wald1_rej_prob, length(delta), length(K)*length(eta))';
Wald2_rej_prob = reshape(Wald2_rej_prob, length(delta), length(K)*length(eta))';

% Print the result
fprintf('\n');
fprintf('                                    [ Pretest (J-test) rejection probability ]\n');
fprintf('---------------||---------------- CASE1 ----------------||---------------- CASE2 ----------------\n');
fprintf('   (K / eta)   ||   (2/.05)     (2/1)  (10/.05)    (10/1)   (2/.05)     (2/1)  (10/.05)    (10/1)\n');
fprintf('---------------||--------------------------------------------------------------------------------\n');
fprintf(' delta = %5.2f ||%10.4f%10.4f%10.4f%10.4f%10.4f%10.4f%10.4f%10.4f\n',[delta;J1_rej_prob;J2_rej_prob]);
fprintf('\n');

fprintf('                     [ Wald test rejection probability conditional on pretest not rejected ]\n');
fprintf('---------------||---------------- CASE1 ----------------||---------------- CASE2 ----------------\n');
fprintf('   (K / eta)   ||   (2/.05)     (2/1)  (10/.05)    (10/1)   (2/.05)     (2/1)  (10/.05)    (10/1)\n');
fprintf('---------------||--------------------------------------------------------------------------------\n');
fprintf(' delta = %5.2f ||%10.4f%10.4f%10.4f%10.4f%10.4f%10.4f%10.4f%10.4f\n',[delta;Wald1_rej_prob;Wald2_rej_prob]);
fprintf('\n');

% Plot the result
figure(1);
CN = 0;             % initialize case number
for h=1:length(K)
    for i=1:length(eta)
        CN = CN + 1;        % increase case number
        
        % Case I
        subplot(4,2,2*CN-1);
        plot(delta,J1_rej_prob(CN,:),'b-',delta,Wald1_rej_prob(CN,:),'r--');
        axis([1.2*min(delta) 1.2*max(delta) 0 1.2*max([J1_rej_prob(CN,:),Wald1_rej_prob(CN,:)])]);
        T = sprintf('K=%d,eta=%1.2f,CASE1',K(h),eta(i));
        title(T);           % display the case

        % Case II
        subplot(4,2,2*CN);
        plot(delta,J2_rej_prob(CN,:),'b-',delta,Wald2_rej_prob(CN,:),'r--');
        axis([1.2*min(delta) 1.2*max(delta) 0 1.2*max([J2_rej_prob(CN,:),Wald2_rej_prob(CN,:)])]);
        T = sprintf('K=%d,eta=%1.2f,CASE2',K(h),eta(i));
        title(T);           % display the case
    end
end
legend('Pretest rejection prob','Conditional Wald rej prob');
