%%% Econ 203C (Spring 2009)
%%% Problem Set IV, Question 3
%%% Empirical Exercise using 2SLS estimation and J-test

clear all;              % clear all the variables
clc;                    % clear the screen

% Load the data
card = load('card_data.txt');
wage = card(:,26);      % log wage (cents/1976)
educ = card(:,4);       % education(years)
exper = card(:,29);     % experience(years)
south = card(:,24);     % dummy indicating south (1976)
south66 = card(:,21);   % dummy indicating south (1966)
black = card(:,22);     % dummy indicating black
near4 = card(:,3);      % dummy indicating near to 4 year college
near2 = card(:,2);      % dummy indicating near to 2 year college
fatheduc = card(:,6);   % education of the father(years)
motheduc = card(:,7);   % education of the mother(years)
clear card;             % clear the variable "card"

% Sample size
n = size(wage,1);

% Construct constant term
l = ones(n,1);

% Question (i) OLS estimation
x = [l, educ, exper, exper.^2, south, black];
y = log(wage);
beta_OLS = inv(x'*x)*x'*y;
u_hat = y - x*beta_OLS;
sigma2_hat = u_hat'*u_hat / (n-6);
var_OLS = sigma2_hat * inv(x'*x);
se_OLS = sqrt(diag(var_OLS));
                        % unbiased estimator of SE(beta) under conditional homoskedasticity
var_robust_OLS = inv(x'*x) * (u_hat * ones(1,6) .* x)' * (u_hat * ones(1,6) .* x) * inv(x'*x);
se_robust_OLS = sqrt(diag(var_robust_OLS));
                        % heteroskedasticity robust estimator of SE(beta)

% Question (ii) 2SLS (or exact IV) estimation
z = [l, near4, exper, exper.^2, south, black];
beta_exactIV = inv(z'*x)*z'*y;
u_hat = y - x*beta_exactIV;
sigma2_hat = u_hat'*u_hat / n;
var_exactIV = sigma2_hat * inv(z'*x)*z'*z*inv(x'*z);
se_exactIV = sqrt(diag(var_exactIV));
                        % consistent estimator of SE(beta) under conditional homoskedasticity
var_robust_exactIV = inv(z'*x) * (u_hat * ones(1,6) .* z)' * (u_hat * ones(1,6) .* z) * inv(x'*z);
se_robust_exactIV = sqrt(diag(var_robust_exactIV));
                        % heteroskedasticity robust estimator of SE(beta)

% Question (iii) 2SLS estimation with overidentifying moment conditions
z = [l, exper, exper.^2, south, black, near4, near2, fatheduc, motheduc];
beta_2SLS = inv(x'*z*inv(z'*z)*z'*x)*x'*z*inv(z'*z)*z'*y;
u_hat = y - x*beta_2SLS;
sigma2_hat = u_hat'*u_hat / n;
var_2SLS = sigma2_hat * inv(x'*z*inv(z'*z)*z'*x);
se_2SLS = sqrt(diag(var_2SLS));
                        % consistent estimator of SE(beta) under conditional homoskedasticity
var_robust_2SLS = inv(x'*z*inv(z'*z)*z'*x) * x'*z*inv(z'*z) * ...
    (u_hat * ones(1,9) .* z)' * (u_hat * ones(1,9) .* z) * inv(z'*z)*z'*x * inv(x'*z*inv(z'*z)*z'*x);
se_robust_2SLS = sqrt(diag(var_robust_2SLS));
                        % heteroskedasticity robust estimator of SE(beta)

% Question (iv) Feasible efficient GMM estimation
V_hat = (u_hat * ones(1,9) .* z)' * (u_hat * ones(1,9) .* z) / n;
beta_GMM = inv(x'*z*inv(V_hat)*z'*x)*x'*z*inv(V_hat)*z'*y;
u_hat = y - x*beta_GMM;
var_asym_GMM = inv(x'*z * inv((u_hat * ones(1,9) .* z)' * (u_hat * ones(1,9) .* z)) * z'*x);
se_asym_GMM = sqrt(diag(var_asym_GMM));
                        % approximation using asymptotic variance formula
var_finite_GMM = inv(x'*z*inv(V_hat)*z'*x) * x'*z*inv(V_hat) * ...
    (u_hat * ones(1,9) .* z)' * (u_hat * ones(1,9) .* z) * inv(V_hat)*z'*x * inv(x'*z*inv(V_hat)*z'*x);
se_finite_GMM = sqrt(diag(var_finite_GMM));
                        % finite iteration robust estimator of SE(beta) for step 2 estimator of beta

% Question (v) J-test
Jn3 = n * (y - x*beta_2SLS)'*z/n * inv(V_hat) * z'*(y - x*beta_2SLS)/n;
                        % using 2SLS estimator
Jn4 = n * u_hat'*z/n * inv(V_hat) * z'*u_hat/n;
                        % using efficient GMM estimator
Jn5 = n * u_hat'*z/n * inv((u_hat * ones(1,9) .* z)' * (u_hat * ones(1,9) .* z)/n) * z'*u_hat/n;
                        % using one step further estimator of the middle term

% Print the result
COEFF = ['beta0';'beta1';'beta2';'beta3';'beta4';'beta5'];
fprintf('\n');
fprintf('==============================================================\n');
fprintf('                      OLS     exact IV       2SLS         GMM\n');
fprintf('==============================================================\n');
for i=1:6
    fprintf('%10s  || %10.4f  %10.4f  %10.4f  %10.4f\n',COEFF(i,:),beta_OLS(i),beta_exactIV(i),beta_2SLS(i),beta_GMM(i));
    fprintf('      (SE)  ||    (%6.4f)    (%6.4f)    (%6.4f)    (%6.4f)\n',se_OLS(i),se_exactIV(i),se_2SLS(i),se_asym_GMM(i));
    fprintf('  (robust)  ||    (%6.4f)    (%6.4f)    (%6.4f)    (%6.4f)\n',...
        se_robust_OLS(i),se_robust_exactIV(i),se_robust_2SLS(i),se_finite_GMM(i));
    fprintf('--------------------------------------------------------------\n');
end

REJ = ['NOT REJECTED';'    REJECTED'];
fprintf('\n');
fprintf('==============================================================\n');
fprintf('       Specification          J-statistic      Null rejection\n');
fprintf('==============================================================\n');
fprintf('  beta_2SLS, first V_hat       %10.4f        %12s\n',Jn3,REJ((Jn3 > chi2inv(.95,3))+1,:));
fprintf('  beta_GMM,  first V_hat       %10.4f        %12s\n',Jn4,REJ((Jn4 > chi2inv(.95,3))+1,:));
fprintf('  beta_GMM, second V_hat       %10.4f        %12s\n',Jn5,REJ((Jn5 > chi2inv(.95,3))+1,:));
fprintf('--------------------------------------------------------------\n');
fprintf('  * critical value = chi2 (quantile .95, df 3) :   %10.4f\n',chi2inv(.95,3));
